detect_lvm()
{
    which pvdisplay >/dev/null 2>&1 && pvdisplay $device >/dev/null 2>&1
}

describe_lvm()
{
    echo "    LVM (linear allocation scheme)"
}

do_lvm()
{
    local pvname
    local pesize
    local vgname
    local pestart
    local penum
    local subsector
    local lestart
    local firstpe
    local lenum
    local fsdev
    local fssector

    echo "# $device: LVM detected" 1>&2

    # Step 3: Get pysical extent number

    pvname="`pvdisplay -c $device 2>/dev/null | awk -F: '{print $1,$2,$8}' | sed -e 's/^ *//'`"

    if [ -z "$pvname" ]; then
        echo "! $device is not a physical volume" 1>&2
        exit 1
    fi

    pesize="${pvname##* }"
    vgname="${pvname% *}"
    pvname="${vgname% *}"
    vgname="${vgname#* }"
    pesize="`dc -e "$pesize 2 * p"`" #`"
    pestart="`pvs --unit s -ope_start $device 2>/dev/null | sed -n -e 's/ *//g' -e 's/S.*$//' -e '$p'`"
    penum="`dc -e "$sector $pestart - $pesize ~ n [ ] n p"`" #`"
    subsector="${penum% *}"
    penum="${penum#* }"

    echo "device $pvname pv"
    echo "offset $pestart"
    echo "pesize $pesize"
    echo "extent $penum"
    echo "subsector $subsector"
    echo "group $vgname"

    # Step 4: Find associated logical volume

    lestart="$(vgdisplay -v $vgname 2>/dev/null | awk '/LV Name/{print $3}' | while read lvname; do \
        lvdisplay -m $lvname 2>/dev/null \
            | awk -v RS="\n *\n(  --- Segments ---\n)?" \
                  -F"[ \t\n:]+" \
                  -v pvname="$pvname" \
                  -v penum="$penum" \
                  -v lvname="$lvname" \
                  '$0 ~ "Physical volume[ \t]+" pvname && $14<=penum && $16>=penum{print lvname,$4,$14}'; \
        done)"

    if [ -z "$lestart" ]; then
        echo "# pysical extent $penum of $pvname is not mapped in any logical volume" 1>&2
        exit 0
    fi

    lvname="${lestart%% *}"
    lestart="${lestart#* }"
    firstpe="${lestart#* }"
    lestart="${lestart% *}"
    lenum="`dc -e "$penum $firstpe - $lestart + p"`" #`"

    echo "device $lvname lv"
    echo "extent $lenum"

    fsdev="$lvname"
    fssector="`dc -e "$lenum $pesize * $subsector + p"`" #`"

    echo "sector $fssector"

    device="$fsdev"
    sector="$fssector"

    return 1
}

register_mapper "lvm"
